from pathlib import Path
import tempfile
from math import isclose

from unittest import TestCase
from unittest.mock import patch

from obspy.core import Stream, read as read_ms
import numpy as np

from sohstationviewer.model.handling_data import (
    readSOHMSeed,
    readSOHTrace,
    readMPTrace,
    readWaveformTrace,
)
from sohstationviewer.model.reftek.from_rt2ms.core import Reftek130

TEST_DATA_DIR = Path(__file__).parent.parent.joinpath('test_data')
# tempfile.tempdir = './tempdir'


class TestHandlingData(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        rt130_dir = TEST_DATA_DIR.joinpath(
            'RT130-sample/2017149.92EB/2017150/92EB/9')
        cls.rt130_file = rt130_dir.joinpath('054910000_013EE8A0')
        rt130 = Reftek130.from_file(cls.rt130_file)
        cls.rt130_stream = Reftek130.to_stream(rt130)
        cls.rt130_trace = cls.rt130_stream[0]

        q330_dir = TEST_DATA_DIR.joinpath('Q330-sample/day_vols_AX08')
        cls.q330_file = q330_dir.joinpath('AX08.XA..VKI.2021.186')
        cls.mseed_stream = read_ms(cls.q330_file)
        cls.mseed_trace = cls.mseed_stream[0]

    # @expectedFailure
    # def test_read_sohmseed(self):
    #     self.fail()

    def test_read_soh_trace_processed_trace_have_all_needed_info(self):
        processed_trace = readSOHTrace(self.mseed_trace)
        with self.subTest('test_processed_trace_have_all_needed_info'):
            expected_key_list = [
                'chanID',
                'samplerate',
                'startTmEpoch',
                'endTmEpoch',
                'times',
                'data'
            ]
            self.assertTrue(
                all(key in processed_trace for key in expected_key_list),
            )

    def test_read_soh_trace_times_calculated_correctly(self):
        processed_trace = readSOHTrace(self.mseed_trace)
        if isclose(processed_trace['startTmEpoch'], 0, abs_tol=0.0001):
            self.assertAlmostEqual(processed_trace['times'][0], 0)
        else:
            self.assertNotAlmostEqual(processed_trace['times'][0], 0)

    @patch('sohstationviewer.model.handling_data.readSOHTrace')
    def test_read_mp_trace(self, mock_read_soh_trace):
        mock_read_soh_trace.return_value = {
            # Contain five cases:
            # + Small positive
            # + Large positive
            # + Small negative
            # + Large negative
            # + Zero
            'data': np.array([1, 27272, -2, -23526, 0])
        }
        expected = np.array([0, 8.3, 0, -7.2, 0])
        processed_trace = readMPTrace(self.rt130_trace)
        self.assertTrue(
            np.array_equal(processed_trace['data'], expected)
        )

    @patch('sohstationviewer.model.handling_data.saveData2File')
    def test_read_waveform_trace(self, mock_save_data_2_file):
        station_id = self.rt130_trace.stats['station']
        channel_id = self.rt130_trace.stats['channel']
        # The function itself only cares about the length of this list so we
        # stub it out.
        traces_info = [dict() for _ in range(4)]
        tmp_dir = tempfile.TemporaryDirectory()
        processed_trace = readWaveformTrace(
            self.rt130_trace, station_id, channel_id, traces_info, tmp_dir.name
        )

        expected_key_list = [
            'samplerate',
            'startTmEpoch',
            'endTmEpoch',
            'size',
            'times_f',
            'data_f',
        ]
        self.assertTrue(
            all(key in processed_trace for key in expected_key_list),
        )

        self.assertTrue(mock_save_data_2_file.called)

    # @skip
    # def test_read_waveform_mseed(self):
    #     self.fail()
    #
    # @skip
    # def test_read_waveform_reftek(self):
    #     self.fail()
    #
    # @skip
    # def test_read_ascii(self):
    #     self.fail()
    #
    # @skip
    # def test_read_text(self):
    #     self.fail()
    #
    # @skip
    # def test_save_data2file(self):
    #     self.fail()
    #
    # @skip
    # def test_check_chan(self):
    #     self.fail()
    #
    # @skip
    # def test_check_sohchan(self):
    #     self.fail()
    #
    # @skip
    # def test_check_wfchan(self):
    #     self.fail()
    #
    # @skip
    # def test_sort_data(self):
    #     self.fail()
    #
    # @skip
    # def test_squash_gaps(self):
    #     self.fail()
    #
    # @skip
    # def test_downsample(self):
    #     self.fail()
    #
    # @skip
    # def test_constant_rate(self):
    #     self.fail()
    #
    # @skip
    # def test_chunk_minmax(self):
    #     self.fail()
    #
    # @skip
    # def test_trim_downsample_sohchan(self):
    #     self.fail()
    #
    # @skip
    # def test_trim_downsample_wfchan(self):
    #     self.fail()
    #
    # @skip
    # def test_get_each_day5min_list(self):
    #     self.fail()
    #
    # @skip
    # def test_get_trim_tpsdata(self):
    #     self.fail()
    #
    # @skip
    # def test_find_tpstm(self):
    #     self.fail()
